!!
!! This module contains public physical constantes and public tools functions like
!! qsat, dev_qsat
!!
!! @call qsfrict_init
!!
!! @author Marie-Alice Foujols and Jan Polcher
!! @Version : $Revision: 1.20.2.1 $, $Date: 2004/01/13 10:59:43 $
!!
!! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_parameters/constantes_veg.f90,v 1.20.2.1 2004/01/13 10:59:43 ssipsl Exp $
!!
!-
MODULE constantes_veg
!---------------------------------------------------------------------
  USE reqdprec
  USE constantes
!-
  IMPLICIT NONE
!-
  LOGICAL, SAVE                    :: l_qsat_first=.TRUE.
!-
! Flags that (de)activate parts of the model
!-
  TYPE(control_type),SAVE :: control
!-
  INTEGER(short), PARAMETER        :: nvm=13              !! Number of vegetation types
!-
  INTEGER(short), PARAMETER        :: nnobio=1            !! Number of other surface types: land ice (lakes,cities, ...)
!-
  INTEGER(short), PARAMETER        :: ngrnd=7             !! Number of soil level
!-
  INTEGER(short), PARAMETER        :: nbdl=6              !! Number of diagnostic levels in the soil
!-
  INTEGER(short), PARAMETER        :: nbsoil=3            !! Number of soil types
!-
  INTEGER(short), PARAMETER        :: iice = 1            !! Index for land ice (see nnobio)
  REAL(stnd), PARAMETER            :: maxmass_glacier = 1.E+6  !! The maximum mass (kg/m^2) of a glacier.
!-
  REAL(stnd), PARAMETER            :: zero = 0._stnd           !! Numerical constant set to 0
  REAL(stnd), PARAMETER            :: min_sechiba = 1.E-8_stnd !! Epsilon to detect a near zero floating point
  REAL(stnd), PARAMETER            :: undef_sechiba = 1.E+20_stnd !! The undef value used in SECHIBA
  REAL(stnd), PARAMETER            :: undemi = 0.5_stnd        !! Numerical constant set to 1/2
  REAL(stnd), PARAMETER            :: un = 1._stnd             !! Numerical constant set to 1
  REAL(stnd), PARAMETER            :: deux = 2._stnd           !! Numerical constant set to 2
  REAL(stnd), PARAMETER            :: trois = 3._stnd          !! Numerical constant set to 3
  REAL(stnd), PARAMETER            :: quatre = 4._stnd         !! Numerical constant set to 4
  REAL(stnd), PARAMETER            :: cinq = 5._stnd           !! Numerical constant set to 5
!-
  REAL(stnd), PARAMETER            :: min_vegfrac=0.001   !! Minimal fraction of mesh a vegetation type can occupy
!-
  REAL(stnd), PARAMETER            :: defc_plus=23.E-3   !! Constant in the computation of surface resistance
!-
  REAL(stnd), PARAMETER            :: defc_mult=1.5      !! Constant in the computation of surface resistance
!-
  REAL(stnd), PARAMETER            :: tsnow=273.         !! Limit of air temperature for snow
!-
  REAL(stnd), PARAMETER            :: snowcri=1.5        !! Sets the amount above which only sublimation occures [Kg/m^2]

  REAL(stnd), PARAMETER            :: snowcri_alb=10.        !! critical value for computation of snow albedo [Kg/m^2]

  REAL(stnd), PARAMETER            :: sneige=snowcri/1000._stnd !! Lower limit of snow amount

  REAL(stnd), PARAMETER            :: pi=3.141592653589793238   !! No comment

  REAL(stnd), PARAMETER            :: tp_00=273.15        !!  0 degre Celsius in degre Kelvin

  REAL(stnd), PARAMETER            :: chalsu0 = 2.8345E06 !! Latent heat of sublimation

  REAL(stnd), PARAMETER            :: chalev0 = 2.5008E06 !! Latent heat of evaporation

  REAL(stnd), PARAMETER            :: chalev1 = 2.5008E06 !! Latent heat of evaporation 2 (?)

  REAL(stnd), PARAMETER            :: chalfu0 = chalsu0 - chalev0 !! Latent heat of fusion
  !
  !     Constantes from HYDROL
  !
  REAL(stnd), PARAMETER            :: qwilt = 5.0        !! Wilting point (Has a numerical role for the moment)

  REAL(stnd), PARAMETER            :: dpu=2.             !! Total depth of soil reservoir

  REAL(stnd), PARAMETER            :: min_resdis =2.e-5  !! The minimal size we allow for the upper reservoir (m)

  REAL(stnd), PARAMETER            :: min_drain = 0.001  !! Diffusion constant for the slow regime (This is for the
                                                                                                            !! diffusion between reservoirs)
  REAL(stnd), PARAMETER            :: max_drain = 0.1  !! Diffusion constant for the fast regime

  REAL(stnd), PARAMETER            :: exp_drain = 1.5   !! The exponential in the diffusion law

  REAL(stnd), PARAMETER            :: mx_eau_eau = 150. !! Quantite maximale d'eau en Kg/M3
  !
  !
  !
  REAL(stnd), PARAMETER            :: c_stefan = 5.6697E-8   !! Stefan-Boltzman constant

  REAL(stnd), PARAMETER            :: cp_air = 1004.675  !! Specific heat of air

  REAL(stnd), PARAMETER            :: cte_molr = 287.05  !! Constante molere

  REAL(stnd), PARAMETER            :: kappa = cte_molr / cp_air    !! Kappa

  REAL(stnd), PARAMETER            :: msmlr_air = 28.964E-03   !! in -- Kg/mole

  REAL(stnd), PARAMETER            :: msmlr_h2o = 18.02E-03    !! in -- Kg/mole

  REAL(stnd), PARAMETER            :: cp_h2o = cp_air * (4._stnd * msmlr_air ) / ( 3.5_stnd * msmlr_h2o ) !!

  REAL(stnd), PARAMETER            :: cte_molr_h2o = cte_molr / 4._stnd   !!

  REAL(stnd), PARAMETER            :: retv = msmlr_air / msmlr_h2o - 1._stnd  !!

  REAL(stnd), PARAMETER            :: rvtmp2 = cp_h2o / cp_air - 1._stnd  !!

  REAL(stnd), PARAMETER            :: cepdu2 = (0.1_stnd) **2             !!

  REAL(stnd), PARAMETER            :: ct_karman = 0.35_stnd               !! Van Karmann Constante

  REAL(stnd), PARAMETER            :: cte_grav = 9.80665_stnd             !! g acceleration

  REAL(stnd), PARAMETER            :: cb = 5._stnd                        !! Constantes of the Louis scheme

  REAL(stnd), PARAMETER            :: cc = 5._stnd                        !! Constantes of the Louis scheme

  REAL(stnd), PARAMETER            :: cd = 5._stnd                        !! Constantes of the Louis scheme

  REAL(stnd), PARAMETER            :: min_wind = 0.1                      !! The minimum wind

  REAL(stnd), PARAMETER            :: pa_par_hpa = 100._stnd   !! Transform pascal into hectopascal

  REAL(stnd), PARAMETER            :: tcst_snowa = 5._stnd     !! Time constant of the albedo decay of snow

  REAL(stnd), PARAMETER            :: max_snow_age = 50._stnd  !! Maximum period of snow aging

  REAL(stnd), PARAMETER            :: snow_trans = 0.3_stnd    !! Transformation time constant for snow (m)

  REAL(stnd), PARAMETER            :: z0_bare = 0.01           !! bare soil roughness length (m)

  REAL(stnd), PARAMETER            :: z0_ice = 0.001           !! ice roughness length (m)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !!
  !! The following tables of parameters for SECHIBA are in the following order :
  !!
  !!    1 - Bare soil
  !!    2 - tropical  broad-leaved evergreen
  !!    3 - tropical  broad-leaved raingreen
  !!    4 - temperate needleleaf   evergreen
  !!    5 - temperate broad-leaved evergreen
  !!    6 - temperate broad-leaved summergreen
  !!    7 - boreal    needleleaf   evergreen
  !!    8 - boreal    broad-leaved summergreen
  !!    9 - boreal    needleleaf   summergreen
  !!   10 -           C3           grass
  !!   11 -           C4           grass
  !!   12 -           C3           agriculture
  !!   13 -           C4           agriculture
  !!
  !! Value for veget_ori for tests in 0-dim simulations
  !! veget_ori_fixed_test_1 = 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0
  REAL(stnd), DIMENSION(nvm), SAVE :: veget_ori_fixed_test_1 = (/0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)

  !! Value for frac_nobio for tests in 0-dim simulations
  !! frac_nobio_fixed_test_1 = 0.0
  ! laisser ca tant qu'il n'y a que de la glace (pa de lacs)
  REAL(stnd), SAVE :: frac_nobio_fixed_test_1 = 0.0
  ! REAL(stnd), DIMENSION(nnobio), SAVE :: frac_nobio_fixed_test_1 = (/0.0 /)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! laimax for maximum lai see also type of lai interpolation
  !! llaimax = 0., 8., 8., 4., 4.5, 4.5, 4., 4.5, 4., 2., 2., 2., 2.
  REAL(stnd), DIMENSION(nvm), SAVE :: llaimax = (/0., 8., 8., 4., 4.5, 4.5, 4., 4.5, 4., 2., 2., 2., 2./)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! laimin for minimum lai see also type of lai interpolation
  !! llaimin = 0., 8., 0., 4., 4.5, 0., 4., 0., 0., 0., 0., 0., 0.
  REAL(stnd), DIMENSION(nvm), SAVE :: llaimin = (/0., 8., 0., 4., 4.5, 0., 4., 0., 0., 0., 0., 0., 0./)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! prescribed height of vegetation.
  !! Value for height_presc : one for each vegetation type
  !! height_presc =   0.,30.,30.,20.,20.,20.,15.,15.,15.,.5,.6,1.,1.
  REAL(stnd), DIMENSION(nvm), SAVE :: height_presc = (/0.,30.,30.,20.,20.,20.,15.,15.,15.,.5,.6,1.,1./)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! Structural resistance.
  !! Value for rstruct_const : one for each vegetation type
  !! rstruct_const = 0.0, 25.0, 25.0, 25.0, 25.0, 25.0, 25.0, 25.0, 25.0, 2.5, 2.0, 2.0, 2.0
  REAL(stnd), DIMENSION(nvm), SAVE :: rstruct_const = (/0.0, 25.0, 25.0, 25.0, 25.0, 25.0, 25.0,&
       & 25.0, 25.0, 2.5, 2.0, 2.0, 2.0 /)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! A vegetation dependent constant used in the calculation of the surface resistance.
  !! Value for kzero one for each vegetation type
  !! kzero =  0.0, 12.E-5, 12.E-5, 12.e-5, 12.e-5, 25.e-5, 12.e-5, 25.e-5, 25.e-5, 30.e-5, 30.e-5, 30.e-5, 30.e-5
  REAL(stnd), DIMENSION(nvm), SAVE :: kzero = (/0.0, 12.E-5, 12.E-5, 12.e-5, 12.e-5, 25.e-5, 12.e-5,&
       & 25.e-5, 25.e-5, 30.e-5, 30.e-5, 30.e-5, 30.e-5 /)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! Maximum field capacity for each of the vegetations (Temporary).
  !! Value of wmax_veg : max quantity of water : one for each vegetation type en Kg/M3
  !! wmax_veg = 150., 150., 150., 150., 150., 150., 150., 150., 150., 150., 150., 150., 150.
  REAL(stnd), DIMENSION(nvm), SAVE :: wmax_veg = (/150., 150., 150., 150., 150., 150., 150.,&
       & 150., 150., 150., 150., 150., 150./)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! Root profile description for the different vegetation types.
  !! These are the factor in the exponential which gets the root density as a function of depth
  !! humcste = 5., .8, .8, 1., .8, .8, 1., 1., .8, 4., 4., 4., 4.
  REAL(stnd), DIMENSION(nvm), SAVE :: humcste = (/0., .8, .8, 1., .8, .8, 1., 1., .8, 4., 4., 4., 4./)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! Type of behaviour of the LAI evolution algorithm for each vegetation type.
  !! Value of type_of_lai, one for each vegetation type : mean or interp
  !! type_of_lai = 'mean ', 'mean', 'inter', 'mean', 'mean', 'inter', 'mean', 'inter', 'inter', 'inter', 'inter', 'inter', 'inter'
  CHARACTER(len=5), DIMENSION(nvm), SAVE :: type_of_lai = (/'mean ', 'mean ', 'inter', 'mean ', 'mean ',&
       & 'inter', 'mean ',  'inter', 'inter', 'inter', 'inter', 'inter', 'inter' /)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! Is the vegetation type a tree ?
  !! is_tree = .FALSE., .TRUE., .TRUE., .TRUE., .TRUE., .TRUE., .TRUE., .TRUE., .TRUE., FALSE., .FALSE., .FALSE., .FALSE.
  LOGICAL, DIMENSION(nvm), SAVE :: is_tree = (/.FALSE., .TRUE., .TRUE., .TRUE., .TRUE., &
              .TRUE., .TRUE., .TRUE., .TRUE., .FALSE., .FALSE., .FALSE., .FALSE./)

  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! Initial snow albedo value for each vegetation type as it will be used in condveg_snow
  !! Values are from the Thesis of S. Chalita (1992)
  !! snowa_ini =  0.55, 0., 0., 0.14, 0.15, 0.15, 0.14, 0.15, 0.14, 0.18, 0.18, 0.18, 0.18
  REAL(stnd), DIMENSION(nvm), SAVE :: snowa_ini = (/0.55, 0., 0., 0.14, 0.15, 0.15, 0.14, 0.15, 0.14, 0.18, 0.18, 0.18, 0.18/)
  ! duplicate initialization to keep a trace in HTML auto generated file with f90doc
  !! Decay rate of snow albedo value for each vegetation type as it will be used in condveg_snow
  !! Values are from the Thesis of S. Chalita (1992)
  !! snowa_dec = 0.30, 0., 0., 0.06, 0.14, 0.14, 0.06, 0.25, 0.06, 0.63, 0.63, 0.63, 0.63
  REAL(stnd), DIMENSION(nvm), SAVE :: snowa_dec = (/0.30, 0., 0., 0.06, 0.14, 0.14, 0.06, 0.25, 0.06, 0.63, 0.63, 0.63, 0.63/)

  !! leaf albedo of vegetation type, VIS+NIR
  !! alb_leaf(:,1) = .00, .10, .10, .07, .10, .10, .07, .10, .07, .11, .11, .11, .11
  !! alb_leaf(:,2) = .00, .45, .45, .35, .45, .45, .35, .45, .35, .58, .58, .58, .58
  REAL(stnd), DIMENSION(nvm*2), SAVE :: &
       !alb_leaf = (/.00, .04, .06, .05, .04, .08, .10, .08, .09, .10, .10, .10, .10, &
       !             .00, .20, .22, .23, .20, .28, .30, .28, .29, .30, .30, .30, .30   /)
       alb_leaf = (/.00, .04, .06, .06, .06, .06, .06, .06, .06, .10, .10, .10, .10, &
                    .00, .20, .22, .22, .22, .22, .22, .22, .22, .30, .30, .30, .30   /)

  !! Default soil texture distribution in the following order : silt, sand and clay
  REAL(stnd), DIMENSION(nbsoil), SAVE :: soiltype_default = (/ 0.0, 1.0, 0.0 /)

  !! albedo of dead leaves, VIS+NIR
  REAL(stnd), DIMENSION(2), SAVE :: alb_deadleaf = (/ .12, .35/)

  !! albedo of ice, VIS+NIR
  REAL(stnd), DIMENSION(2), SAVE :: alb_ice = (/ .60, .20/)

  !! The lower limit of the layer on which soil moisture (relative) and temopoerature
  !! are going to be diagnosed. These variables are made for trnasfering the information
  !! to the biogeophyical processes modelled in STOMATE.
  !!
  REAL(stnd), DIMENSION(nbdl), SAVE :: diaglev = (/0.05, 0.1, 0.2, 0.5, 1.0, 2.0/)

  REAL(stnd), PARAMETER            :: hcrit_litter=0.03  !! scaling depth for litter humidity (m)

  LOGICAL, PARAMETER               :: ldveget_ori_on_restart = .TRUE.  !! Is veget_ori array stored in restart file

  LOGICAL, PARAMETER               :: ldq_cdrag_from_gcm = .FALSE. !! Set to .TRUE. if you want q_cdrag coming from GCM

  !! Constant in the computation of resistance for bare  soil evaporation
  REAL(stnd), SAVE            :: rsol_cste = 33.E3
  REAL(stnd), SAVE            :: rveg_cste = 1.
  REAL(stnd), SAVE            :: qsintcst = 0.01


  REAL(stnd), PARAMETER            :: rayt_cste = 125.   !! Constant in the computation of surface resistance


  ! constantes for thermosoil : to be keeped here or not?

  REAL(stnd), PARAMETER            :: un_jour = 86400.   !! One day in seconds

  REAL(stnd), PARAMETER            :: so_cond = 1.5396      !! Average Thermal Conductivity of soils
  REAL(stnd), PARAMETER            :: so_capa = 2.0514e+6   !! Average Heat capacity of soils

  ! Values taken from : PIELKE, 'MESOSCALE METEOROLOGICAL MODELING', P. 384
  ! Dry soil heat capacity was decreased and conductivity increased.
!
!  REAL(stnd), PARAMETER            :: so_capa_dry = 1.35e+6 !! Dry soil Heat capacity of soils
!  REAL(stnd), PARAMETER            :: so_cond_dry = 0.28    !! Dry soil Thermal Conductivity of soils
!
  REAL(stnd), PARAMETER            :: so_capa_dry = 1.80e+6 !! Dry soil Heat capacity of soils
  REAL(stnd), PARAMETER            :: so_cond_dry = 0.40    !! Dry soil Thermal Conductivity of soils

  REAL(stnd), PARAMETER            :: so_capa_wet = 3.03e+6 !! Wet soil Heat capacity of soils
  REAL(stnd), PARAMETER            :: so_cond_wet = 1.89    !! Wet soil Thermal Conductivity of soils

  REAL(stnd), PARAMETER            :: sn_cond = 0.3 !! Thermal Conductivity of snow

  REAL(stnd), PARAMETER            :: sn_dens = 330.0  !! Snow density for the soil thermodynamics

  REAL(stnd), PARAMETER            :: sn_capa = 2100.0_stnd * sn_dens !! Heat capacity for snow

  !! Size of local array to keep saturated humidity at each temperature level
  INTEGER(long), PARAMETER                                  :: max_temp=370
  !! Minimum temperature for saturated humidity
  INTEGER(long), PARAMETER                                  :: min_temp=100
  !! Local array to keep saturated humidity at each temperature level
  REAL(stnd), DIMENSION(max_temp), SAVE                     :: qsfrict

CONTAINS

  SUBROUTINE qsatcalc (kjpindex, temp_in, pres_in, qsat_out)

    !
    ! interface description:
    ! input value
    INTEGER(long), INTENT(in)                   :: kjpindex    !! Domain size
    REAL(stnd), DIMENSION(kjpindex), INTENT(in) :: temp_in     !! Temperature in degre Kelvin
    REAL(stnd), DIMENSION(kjpindex), INTENT(in) :: pres_in     !! Pressure
    ! output value
    REAL(stnd), DIMENSION(kjpindex), INTENT(out):: qsat_out    !! Result

    ! local variables
    INTEGER(short), DIMENSION(kjpindex)         :: jt
    INTEGER(short)                              :: ji
    REAL(stnd), DIMENSION(kjpindex)             :: zz_a, zz_b, zz_f
    INTEGER(short)                              :: nbad
    INTEGER(short), DIMENSION(1)                :: lo

    IF (l_qsat_first) THEN

        CALL qsfrict_init

        l_qsat_first = .FALSE.

    END IF

    !
    ! 1. computes qsat interpolation into two successive temperature
    !

    jt = INT(temp_in(:))

    nbad = COUNT( jt(:) .GE. max_temp-1 )
    IF ( nbad .GT. 0 ) THEN
        WRITE (*,*) ' qsatcalc: temperature too high at ', &
                    nbad, ' points.'
        IF ( .NOT. diag_qsat) THEN
           STOP 'qsatcalc'
        ELSE
           lo = MAXLOC(temp_in(:))
           WRITE(*,*) 'Maximum temperature ( ', MAXVAL(temp_in) ,') found at ', lo(1)
           WHERE ( jt(:) .GE. max_temp-1 ) jt(:) = max_temp-1
        ENDIF
    END IF

    nbad = COUNT( jt(:) .LE. min_temp )
    IF ( nbad .GT. 0 ) THEN
        WRITE (*,*) ' qsatcalc: temperature too low at ', &
                    nbad, ' points.'
        IF ( .NOT. diag_qsat) THEN
           STOP 'qsatcalc'
        ELSE
           lo = MINLOC(temp_in(:))
           WRITE(*,*) 'Minimum temperature ( ', MINVAL(temp_in) ,') found at ', lo(1)
           WHERE ( jt(:) .LE. min_temp ) jt(:) = min_temp
        ENDIF
    END IF

    DO ji = 1, kjpindex
      zz_f(ji) = temp_in(ji) - FLOAT(jt(ji))
      zz_a(ji) = qsfrict(jt(ji))
      zz_b(ji) = qsfrict(jt(ji) + 1)
    ENDDO

    !
    ! 2. interpolates between this two values
    !
    DO ji = 1, kjpindex
      qsat_out(ji) = ( (zz_b(ji) - zz_a(ji)) * zz_f(ji) + zz_a(ji) ) / pres_in(ji)
    ENDDO

  END SUBROUTINE qsatcalc

  ! duplicate RESULT to keep a trace in HTML auto generated file with f90doc
  !! FUNCTION qsat (temp_in, pres_in) RESULT (qsat_result)
  !!
  FUNCTION qsat (temp_in, pres_in) RESULT (qsat_result)

    !
    ! interface description:
    ! input value
    REAL(stnd), INTENT(in)                      :: temp_in     !! Temperature in degre Kelvin
    REAL(stnd), INTENT(in)                      :: pres_in     !! Pressure
    ! output value
    REAL(stnd)                                  :: qsat_result !! Result

    ! local variables
    INTEGER(short)                 :: jt
    REAL(stnd)                     :: zz_a, zz_b, zz_f

    IF (l_qsat_first) THEN

        CALL qsfrict_init

        l_qsat_first = .FALSE.

    END IF

    !
    ! 1. computes qsat interpolation into two successive temperature
    !

    jt = INT(temp_in)

    IF (jt .GE. max_temp-1) THEN
        WRITE (*,*) ' We stop. temperature too BIG : ', temp_in, ' approximation for : ', jt
        IF ( .NOT. diag_qsat) THEN
           STOP 'qsat'
        ELSE
           qsat_result = 999999.
           RETURN
        ENDIF
    END IF

    IF (jt .LE. min_temp ) THEN
        WRITE (*,*) ' We stop. temperature too SMALL : ', temp_in, ' approximation for : ', jt
        IF ( .NOT. diag_qsat) THEN
           STOP 'qsat'
        ELSE
           qsat_result = -999999.
           RETURN
        ENDIF
    END IF


    zz_f = temp_in - FLOAT(jt)
    zz_a = qsfrict(jt)
    zz_b = qsfrict(jt + 1)

    !
    ! 2. interpolates between this two values
    !
    qsat_result = ( (zz_b - zz_a) * zz_f + zz_a ) / pres_in

  END FUNCTION qsat

  SUBROUTINE dev_qsatcalc (kjpindex, temp_in, pres_in, dev_qsat_out)

    !
    ! interface description:
    ! input value
    INTEGER(long), INTENT(in)                   :: kjpindex    !! Domain size
    REAL(stnd), DIMENSION(kjpindex), INTENT(in) :: temp_in     !! Temperature in degre Kelvin
    REAL(stnd), DIMENSION(kjpindex), INTENT(in) :: pres_in     !! Pressure
    ! output value
    REAL(stnd), DIMENSION(kjpindex), INTENT(out):: dev_qsat_out!! Result

    ! local variables
    INTEGER(short), DIMENSION(kjpindex)         :: jt
    INTEGER(short)                              :: ji
    REAL(stnd), DIMENSION(kjpindex)             :: zz_a, zz_b, zz_c, zz_f
    INTEGER(short)                              :: nbad

    IF (l_qsat_first) THEN

        CALL qsfrict_init

        l_qsat_first = .FALSE.

    END IF

    !
    ! 1. computes qsat interpolation into two successive temperature
    !

    jt = INT(temp_in(:)+undemi)

    nbad = COUNT( jt(:) .GE. max_temp-1 )
    IF ( nbad .GT. 0 ) THEN
        WRITE (*,*) ' dev_qsatcalc: temperature too high at ', &
                    nbad, ' points.'
        IF ( .NOT. diag_qsat) THEN
           STOP 'dev_qsatcalc'
        ELSE
           WHERE ( jt(:) .GE. max_temp-1 ) jt(:) = max_temp-1
        ENDIF
    END IF

    nbad = COUNT( jt(:) .LE. min_temp )
    IF ( nbad .GT. 0 ) THEN
        WRITE (*,*) ' dev_qsatcalc: temperature too low at ', &
                    nbad, ' points.'
        IF ( .NOT. diag_qsat) THEN
           STOP 'dev_qsatcalc'
        ELSE
           WHERE ( jt(:) .LE. min_temp ) jt(:) = min_temp
        ENDIF
    END IF

    DO ji = 1, kjpindex
      zz_f(ji) = temp_in(ji) + undemi - FLOAT(jt(ji))
      zz_a(ji) = qsfrict(jt(ji)-1)
      zz_b(ji) = qsfrict(jt(ji))
      zz_c(ji) = qsfrict(jt(ji)+1)
    ENDDO

    !
    ! 2. interpolates between this two values
    !
    DO ji = 1, kjpindex
      dev_qsat_out(ji) = ( (zz_c(ji) - deux * zz_b(ji) + zz_a(ji)) * (zz_f(ji) - un) + &
                           zz_c(ji) - zz_b(ji)  ) / pres_in(ji)
    ENDDO

  END SUBROUTINE dev_qsatcalc

  ! duplicate RESULT to keep a trace in HTML auto generated file with f90doc
  !! FUNCTION dev_qsat (temp_in, pres_in) RESULT (dev_qsat_result)
  !! computes deviation of qsat
  !!
  FUNCTION dev_qsat (temp_in, pres_in) RESULT (dev_qsat_result)

    ! interface description:
    ! input value
    REAL(stnd), INTENT(in)                      :: pres_in           !! Pressure
    REAL(stnd), INTENT(in)                      :: temp_in           !! Temperture in degre Kelvin
    ! output value
    REAL(stnd)                                  :: dev_qsat_result

    ! local variables
    INTEGER(short)                 :: jt
    REAL(stnd)                     :: zz_a, zz_b, zz_c, zz_f

    IF (l_qsat_first) THEN

        CALL qsfrict_init

        l_qsat_first = .FALSE.

    END IF

    !
    ! 1. computes qsat deviation interpolation into two successive temperature
    !

    jt = INT(temp_in+undemi)

    IF (jt .GE. max_temp-1) THEN
        WRITE (*,*) ' We stop. temperature too BIG : ', temp_in, ' approximation for : ', jt
        IF ( .NOT. diag_qsat) THEN
           STOP 'dev_qsat'
        ELSE
           dev_qsat_result = 999999.
           RETURN
        ENDIF
    END IF

    IF (jt .LE. min_temp ) THEN
        WRITE (*,*) ' We stop. temperature too SMALL : ', temp_in, ' approximation for : ', jt
        IF ( .NOT. diag_qsat) THEN
           STOP 'dev_qsat'
        ELSE
           dev_qsat_result = -999999.
           RETURN
        ENDIF
    END IF

    zz_f = temp_in + undemi - FLOAT(jt)
    zz_a = qsfrict(jt - 1)
    zz_b = qsfrict(jt)
    zz_c = qsfrict(jt + 1)

    !
    ! 2. interpolates
    !
    dev_qsat_result = ( (zz_c - deux * zz_b + zz_a) * (zz_f - un) + zz_c - zz_b  ) / pres_in

  END FUNCTION dev_qsat

  !!
  !! this function initialises qsfrict array to store precalculated value for qsat
  !!
  SUBROUTINE qsfrict_init

    !
    ! interface description:
    ! input value : none
    ! output value : none

    ! local declaration

    INTEGER(short)                 :: ji
    REAL(stnd)                     :: zrapp, zcorr, ztemperature, zqsat


    ! initialisation

    zrapp = msmlr_h2o / msmlr_air
    zcorr = 0.00320991_stnd

    ! computes saturated humidity one time and store in qsfrict local array

    DO ji=100,max_temp

      ztemperature = FLOAT(ji)

      IF (ztemperature.LT.273._stnd) THEN
          zqsat = zrapp * 10.0_stnd ** (2.07023_stnd - zcorr * ztemperature&
             & - 2484.896 / ztemperature + 3.56654 * LOG10(ztemperature))
      ELSE
          zqsat = zrapp * 10.0 ** (23.8319 - 2948.964 / ztemperature - 5.028 * LOG10(ztemperature)&
             & - 29810.16 * EXP( - 0.0699382 * ztemperature) + 25.21935 * EXP( - 2999.924 / ztemperature))
      ENDIF

      qsfrict (ji) = zqsat

    END DO

    DO ji=1,100

      qsfrict (ji) = zero

    END DO


    IF (long_print) WRITE (numout,*) ' qsfrict_init done '

  END SUBROUTINE qsfrict_init

  ! duplicate RESULT to keep a trace in HTML auto generated file with f90doc
  !! FUNCTION tempfunc (temp_in) RESULT (tempfunc_result)
  !! this function interpolates value between ztempmin and ztempmax
  !! used for lai detection

  FUNCTION tempfunc (temp_in) RESULT (tempfunc_result)

    !
    ! interface description:
    ! input value
    REAL(stnd), INTENT(in)                     :: temp_in            !! Temperature in degre Kelvin
    ! output value
    REAL(stnd)                                 :: tempfunc_result    !! Result

    ! local variables
    REAL(stnd), PARAMETER          :: ztempmin = 273._stnd           !! Temperature for laimin
    REAL(stnd), PARAMETER          :: ztempmax = 293._stnd           !! Temperature for laimax
    REAL(stnd)                     :: zfacteur                       !! Interpolation factor

    zfacteur = un / ( ztempmax - ztempmin) ** 2

    IF ( temp_in .GT. ztempmax) THEN
        tempfunc_result = un
    ELSEIF (temp_in .LT. ztempmin) THEN
        tempfunc_result = zero
    ELSE
        tempfunc_result = un - zfacteur * ( ztempmax - temp_in ) ** 2
    ENDIF

    RETURN

  END FUNCTION tempfunc

  SUBROUTINE get_vegcorr (nolson,vegcorr,nobiocorr)

    INTEGER(long), INTENT(in)                         :: nolson
    REAL(stnd), DIMENSION(nolson,nvm), INTENT(out)    :: vegcorr(nolson,nvm)
    REAL(stnd), DIMENSION(nolson,nnobio), INTENT(out) :: nobiocorr

    INTEGER(long)                                     :: ib
    INTEGER(long), PARAMETER                          :: nolson94 = 94
    INTEGER(long), PARAMETER                          :: nvm13 = 13

    IF ( nolson .NE. nolson94 ) THEN
      WRITE(numout,*) nolson,nolson94
      STOP 'wrong number of OLSON vegetation types'
    ENDIF
    IF ( nvm .NE. nvm13 ) THEN
      WRITE(numout,*) nvm,nvm13
      STOP 'wrong number of SECHIBA vegetation types'
    ENDIF

    !
    ! 1 set the indices of non-biospheric surface types to 0.
    !

    nobiocorr(:,:) = 0.

    !
    ! 2 Here we construct the correspondance table between Olson and the following SECHIBA
    !   Classes.
    !   vegcorr(i,:) + nobiocorr(i,:) = 1.  for all i.
    !
    ! The modified OLSON types found in file carteveg5km.nc created by Nicolas Viovy :
    ! 1  Urban
    vegcorr( 1,:) = (/ 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 2 Cool low sparse grassland
    vegcorr( 2,:) = (/ 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)
    ! 3 Cold conifer forest
    vegcorr( 3,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 4 Cold deciduous conifer forest
    vegcorr( 4,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 5 Cool Deciduous broadleaf forest
    vegcorr( 5,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 6 Cool evergreen broadleaf forests
    vegcorr( 6,:) = (/ 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 7 Cool tall grasses and shrubs
    vegcorr( 7,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)
    ! 8 Warm C3 tall grasses and shrubs
    vegcorr( 8,:) = (/ 0.1, 0.0, 0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)
    ! 9 Warm C4 tall grases and shrubs
    vegcorr( 9,:) = (/ 0.1, 0.0, 0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0 /)
    ! 10 Bare desert
    vegcorr(10,:) = (/ 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 11 Cold upland tundra
    vegcorr(11,:) = (/ 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)
    ! 12 Cool irrigated grassland
    vegcorr(12,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.9, 0.0, 0.0, 0.0 /)
    ! 13 Semi desert
    vegcorr(13,:) = (/ 0.7, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 0.0, 0.0 /)
    ! 14 Glacier ice
    vegcorr(14,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    nobiocorr(14,iice) = 1.
    ! 15 Warm wooded wet swamp
    vegcorr(15,:) = (/ 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0 /)
    ! 16 Inland water
    vegcorr(16,:) = (/ 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 17 sea water
    vegcorr(17,:) = (/ 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 18 cool shrub evergreen
    vegcorr(18,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.3, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0, 0.0, 0.0 /)
    ! 19 cold shrub deciduous
    vegcorr(19,:) = (/ 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.6, 0.0, 0.0, 0.0 /)
    ! 20 Cold evergreen forest and fields
    vegcorr(20,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.5, 0.0, 0.0, 0.0, 0.0, 0.5, 0.0, 0.0, 0.0 /)
    ! 21 cool rain forest
    vegcorr(21,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0 /)
    ! 22 cold conifer boreal forest
    vegcorr(22,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0 /)
    ! 23 cool conifer forest
    vegcorr(23,:) = (/ 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0 /)
    ! 24 warm mixed forest
    vegcorr(24,:) = (/ 0.0, 0.4, 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0 /)
    ! 25 cool mixed forest
    vegcorr(25,:) = (/ 0.0, 0.0, 0.0, 0.4, 0.0, 0.4, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0 /)
    ! 26 cool broadleaf forest
    vegcorr(26,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.9, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0 /)
    ! 27 cool deciduous broadleaf forest
    vegcorr(27,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.3, 0.5, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0 /)
    ! 28 warm montane tropical forest
    vegcorr(28,:) = (/ 0.0, 0.9, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0 /)
    ! 29 warm seasonal tropical forest
    vegcorr(29,:) = (/ 0.0, 0.5, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 0.0 /)
    ! 30 cool crops and towns
    vegcorr(30,:) = (/ 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0 /)
    ! 31 warm crops and towns
    vegcorr(31,:) = (/ 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8 /)
    ! 32 cool crops and towns
    vegcorr(32,:) = (/ 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0 /)
    ! 33 warm dry tropical woods
    vegcorr(33,:) = (/ 0.2, 0.0, 0.5, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 0.0, 0.0 /)
    ! 34 warm tropical rain forest
    vegcorr(34,:) = (/ 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 35 warm tropical degraded forest
    vegcorr(35,:) = (/ 0.1, 0.6, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 0.0 /)
    ! 36 warm corn and beans cropland
    vegcorr(36,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 /)
    ! 37 cool corn and bean cropland
    vegcorr(37,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 /)
    ! 38 warm rice paddy and field
    vegcorr(38,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 /)
    ! 39 hot irrigated cropland
    vegcorr(39,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0 /)
    ! 40 cool irrigated cropland
    vegcorr(40,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 /)
    ! 41 cold irrigated cropland
    vegcorr(41,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 /)
    ! 42 cool grasses and shrubs
    vegcorr(42,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0, 0.7, 0.0, 0.0, 0.0 /)
    ! 43 hot and mild grasses and shrubs
    vegcorr(43,:) = (/ 0.2, 0.0, 0.1, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0, 0.0 /)
    ! 44 cold grassland
    vegcorr(44,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.9, 0.0, 0.0, 0.0 /)
    ! 45 Savanna (woods) C3
    vegcorr(45,:) = (/ 0.1, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.7, 0.0, 0.0, 0.0 /)
    ! 46 Savanna woods C4
    vegcorr(46,:) = (/ 0.1, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.7, 0.0, 0.0 /)
    ! 47 Mire, bog, fen
    vegcorr(47,:) = (/ 0.1, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.7, 0.0, 0.0, 0.0 /)
    ! 48 Warm marsh wetland
    vegcorr(48,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)
    ! 49 cold marsh wetland
    vegcorr(49,:) = (/ 0.0, 0.0, 0.0, 0.1, 0.1, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)
    ! 50 mediteraean scrub
    vegcorr(50,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)
    ! 51 Cool dry woody scrub
    vegcorr(51,:) = (/ 0.3, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0, 0.0, 0.0 /)
    ! 52 Warm dry evergreen woods
    vegcorr(52,:) = (/ 0.1, 0.9, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 53 Volcanic rocks
    vegcorr(53,:) = (/ 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 54 send desert
    vegcorr(54,:) = (/ 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 55 warm semi desert shrubs
    vegcorr(55,:) = (/ 0.7, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 0.0, 0.0 /)
    ! 56 cool semi desert shrubs
    vegcorr(56,:) = (/ 0.6, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.4, 0.0, 0.0, 0.0 /)
    ! 57 semi desert sage
    vegcorr(57,:) = (/ 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0, 0.0, 0.0 /)
    ! 58 Barren tundra
    vegcorr(58,:) = (/ 0.6, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.4, 0.0, 0.0, 0.0 /)
    ! 59 cool southern hemisphere mixed forest
    vegcorr(59,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.3, 0.0, 0.0, 0.0 /)
    ! 60 cool fields and woods
    vegcorr(60,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0 /)
    ! 61 warm forest and filed
    vegcorr(61,:) = (/ 0.0, 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6 /)
    ! 62 cool forest and field
    vegcorr(62,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0 /)
    ! 63 warm C3 fields and woody savanna
    vegcorr(63,:) = (/ 0.1, 0.0, 0.3, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0 /)
    ! 64 warm C4 fields and woody savanna
    vegcorr(64,:) = (/ 0.1, 0.0, 0.3, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6 /)
    ! 65 cool fields and woody savanna
    vegcorr(65,:) = (/ 0.0, 0.0, 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0 /)
    ! 66 warm succulent and thorn scrub
    vegcorr(66,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0 /)
    ! 67 cold small leaf mixed woods
    vegcorr(67,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.3, 0.0, 0.5, 0.0, 0.0, 0.0 /)
    ! 68 cold deciduous and mixed boreal fores
    vegcorr(68,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.7, 0.0, 0.0, 0.3, 0.0, 0.0, 0.0 /)
    ! 69 cold narrow conifers
    vegcorr(69,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.9, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0 /)
    ! 70 cold wooded tundra
    vegcorr(70,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 0.7, 0.0, 0.0, 0.0 /)
    ! 71 cold heath scrub
    vegcorr(71,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 0.7, 0.0, 0.0, 0.0 /)
    ! 72 Polar and alpine desert
    vegcorr(72,:) = (/ 0.9, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0, 0.0 /)
    ! 73 warm Mangrove
    vegcorr(73,:) = (/ 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 74 cool crop and water mixtures
    vegcorr(74,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.0, 0.3, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0 /)
    ! 75 cool southern hemisphere mixed forest
    vegcorr(75,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.4, 0.4, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0 /)
    ! 76 cool moist eucalyptus
    vegcorr(76,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0 /)
    ! 77 warm rain green tropical forest
    vegcorr(77,:) = (/ 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)
    ! 78 warm C3 woody savanna
    vegcorr(78,:) = (/ 0.0, 0.0, 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0, 0.0, 0.0 /)
    ! 79 warm C4 woody savanna
    vegcorr(79,:) = (/ 0.0, 0.0, 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0, 0.0, 0.0 /)
    ! 80 cool woody savanna
    vegcorr(80,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.4, 0.0, 0.6, 0.0, 0.0, 0.0 /)
    ! 81 cold woody savanna
    vegcorr(81,:) = (/ 0.0, 0.0, 0.0, 0.4, 0.0, 0.0, 0.0, 0.0, 0.0, 0.6, 0.0, 0.0, 0.0 /)
    ! 82 warm broadleaf crops
    vegcorr(82,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.9, 0.0 /)
    ! 83 warm C3 grass crops
    vegcorr(83,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.9, 0.0 /)
    ! 84 warm C4 grass crops
    vegcorr(84,:) = (/ 0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.9 /)
    ! 85 cool grass crops
    vegcorr(85,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 /)
    ! 86 warm C3 crops grass,shrubs
    vegcorr(86,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0 /)
    ! 87 cool crops,grass,shrubs
    vegcorr(87,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.5, 0.0, 0.5, 0.0 /)
    ! 88 warm evergreen tree crop
    vegcorr(88,:) = (/ 0.0, 0.8, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2 /)
    ! 89 cool evergreen tree crop
    vegcorr(89,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0 /)
    ! 90 cold evergreen tree crop
    vegcorr(90,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0 /)
    ! 91 warm deciduous tree crop
    vegcorr(91,:) = (/ 0.0, 0.0, 0.8, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2 /)
    ! 92 cool deciduous tree crop
    vegcorr(92,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0, 0.0, 0.0, 0.2, 0.0 /)
    ! 93 cold deciduous tree crop
    vegcorr(93,:) = (/ 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0, 0.2, 0.0 /)
    ! 94 wet sclerophylic forest
    vegcorr(94,:) = (/ 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 /)

    !
    ! 3 Check the mapping for the Olson types which are going into the
    !   the veget and nobio array.
    !

    DO ib=1,nolson
       IF ( ABS(SUM(vegcorr(ib,:))+SUM(nobiocorr(ib,:))-1.0) .GT. EPSILON(1.0) ) THEN
          WRITE(*,*) 'Wrong correspondance for Olson type :', ib
          STOP 'get_vegcorr'
       ENDIF
    ENDDO

  END SUBROUTINE get_vegcorr

  SUBROUTINE get_soilcorr (nzobler, textfrac_table)

    !
    ! Defines the table of correspondence between the Zobler types and the three texture
    ! types known by SECHIBA & STOMATE : silt, sand and clay
    !

    INTEGER(long), INTENT(in)                          :: nzobler
    REAL(stnd), DIMENSION(nzobler,nbsoil), INTENT(out) :: textfrac_table(nzobler,nbsoil)

    INTEGER(long), PARAMETER                           :: nbtypes_zobler = 7
    INTEGER(long)                                      :: ib

    !
    !

    IF ( nzobler .NE. nbtypes_zobler) THEN
       WRITE(*,*) 'We do not have the correct number of classes in the code for the file'
       STOP 'GET_soilcorr'
    ENDIF

    !
    ! Textural fraction for : silt        sand         clay
    !

    textfrac_table(1,:) = (/  0.12,       0.82,        0.06 /)
    textfrac_table(2,:) = (/  0.32,       0.58,        0.10 /)
    textfrac_table(3,:) = (/  0.39,       0.43,        0.18 /)
    textfrac_table(4,:) = (/  0.15,       0.58,        0.27 /)
    textfrac_table(5,:) = (/  0.34,       0.32,        0.34 /)
    textfrac_table(6,:) = (/  0.00,       1.00,        0.00 /)
    textfrac_table(7,:) = (/  0.39,       0.43,        0.18 /)

    DO ib=1,nzobler
       IF ( ABS(SUM(textfrac_table(ib,:))-1.0) .GT. EPSILON(1.0) ) THEN
          WRITE(*,*) 'Error in the correspondence table, sum is not equal to 1 in', ib
          WRITE(*,*) textfrac_table(ib,:)
          STOP 'GET_soilcorr'
       ENDIF
    ENDDO

  END SUBROUTINE GET_soilcorr

END MODULE constantes_veg


